/*!
  \file   Echantillonnage.C
  \brief  Generation d'echantillons
*/
#include <iostream>
#include "nisp_pc.h"
#include "nisp_util.h"

using namespace std;

/*!
  \fn void Echantillonnage(int np)
  \brief Methode : Monte Carlo, Hypercubes Latins, Quasi Monte Carlo
  \param np nombre de simulations : Monte Carlo, Lhs et QmcSobol
*/
void Echantillonnage(int np) {

  // Initialisation de la "graine" aleatoire
  InitSeed(1); // la graine aaleatoire sera differente a chaque run

  // Definition des parametres incertains
  SetRandomVariable * gvu = new SetRandomVariable();
  // Loi normale de moyenne 5.0 et d'ecart-type 1.5
  gvu->AddRandomVariable(new RandomVariable("Normale",5.,1.5));
  // Loi Uniforme minimun=0, maximum=10
  gvu->AddRandomVariable(new RandomVariable("Uniforme",0.,10.));
  // Loi exponentielle de paramatre 1.0
  gvu->AddRandomVariable(new RandomVariable("Exponentielle",1.0));
  // Loi LogNormale de moyenne 0.2 et d'ecart type de la variable normale 1.0
  gvu->AddRandomVariable(new RandomVariable("LogNormale",0.2,1.0));
  // Loi LogUniforme de minimim = 0.001 et de maximum=10
  gvu->AddRandomVariable(new RandomVariable("LogUniforme",0.01,10.));

  // Generation par np tirages Monte Carlo et sauvegarde
  gvu->BuildSample((char *) "MonteCarlo",np);
  gvu->Save((char *) "MC.dat");

  // Generation par np tirages Hypercube Latins et sauvegarde
  gvu->BuildSample("Lhs",np);
  gvu->Save((char *) "LHS.dat");

  // Generation par Quasi Monte Carlo (sequences de Sobol) de taille np et sauvegarde
  gvu->BuildSample("QmcSobol",np);
  gvu->Save((char *) "QMC.dat");
}

int main(int argc, char *argv[]) {
  if(argc != 2) {
    cout << "Commande < nisp np >" << endl;
    cout << "np : nombre de simulations pour MonteCarlo, Lhs et QmcSobol" << endl;
  }
  else {
    int np=atoi(argv[1]);
    Echantillonnage(np);
  }
}
