/*!
  \file   BuildPolynomialChaos.C
  \brief  Construction d'un polynome de chaos a partir d'un fichier reponses
*/
#include <iostream>
#include "nisp_pc.h" // classes Nisp

using namespace std;
// Modele mathematique : y = x1 * x2
void Exemple1(double x[], double y[]) {
  y[0]=x[0]*x[1];
}

/*!
  \fn void CaseExemple1(int degre)
  \brief Construction d'un polynome de chaos a partir d'un fichier reponses
  \param degre : degre du polynme
*/
void BuildPolynomialChaos(char *fichierPlanStochastique, char *fichierReponses, int degre) {
  // Definition des variables stochastiques : variable aleatoire standardisee
  SetRandomVariable * gx = new SetRandomVariable(fichierPlanStochastique);

  // Polynome de chaos
  PolynomialChaos * pc = new PolynomialChaos(gx);

  // Calcul du chaos polynomial
  pc->SetDegree(degre);
  pc->SetSizeTarget(gx->GetSize());
  pc->ReadTarget(fichierReponses);
  pc->ComputeChaosExpansion(gx,"Integration");

  // Edition des resultats
  cout << "Mean     = " << pc->GetMean() << endl;
  cout << "Variance = " << pc->GetVariance() << endl;
  cout << endl;

  cout << "Indice de sensibilite du 1er ordre"<<endl;
  cout << "    Variable X1 = " << pc->GetIndiceFirstOrder(1)<<endl;
  cout << "    Variable X2 = " << pc->GetIndiceFirstOrder(2)<<endl;
  cout << endl;

  cout << "Indice de sensibilite Totale"<<endl;
  cout << "    Variable X1 = " << pc->GetIndiceTotalOrder(1)<<endl;
  cout << "    Variable X2 = " << pc->GetIndiceTotalOrder(2)<<endl;
}

int main(int argc, char *argv[]) {
  if(argc != 4) {
    cout << "Erreur !! la commande est de la forme << nisp PlanStochastique Reponses degre>>" << endl;
  }
  else {
    BuildPolynomialChaos(argv[1],argv[2], atoi(argv[3]));
  }
}
