/*!
  \file   GenererPlan.C
  \brief  Generer un plan d'experiences
*/
#include <iostream>
#include "nisp_pc.h" // classes Nisp


/*!
  \fn void GenererPlan(int degre)
  \brief Generer un plan d'experiences
  \param degre : degre du polynme
*/
void GenererPlan(int degre) {
  // Definition des variables physiques incertaines
  SetRandomVariable * gu = new SetRandomVariable();            // groupe de variables incertaines
  gu->AddRandomVariable(new RandomVariable("Normale",1.,0.5));  // normale, moyenne = 1., ecart type = 0.5
  gu->AddRandomVariable(new RandomVariable("Uniforme",1.,2.0)); // uniforme, min = 1, max = 2

  // Definition des variables stochastiques : variable aleatoire standardisee
  SetRandomVariable * gx = new SetRandomVariable();             // groupe de variables stochastiques
  gx->AddRandomVariable(new RandomVariable("Normale"));         // normale, moyenne 0., ecart-type 1
  gx->AddRandomVariable(new RandomVariable("Uniforme"));        // uniforme, min = 0, max = 1
  
  // Calcul du plan d'experiences numeriques par Quadrature
  gx->BuildSample("Quadrature",degre);    // calcul des points et poids par quadrature
  gu->BuildSample(gx);                    // calcul du plan d'experiences numeriques

  // Sauvegarde des plans
  gx->Save((char *) "PlanStochastique.dat");
  gu->Save((char *) "PlanExperiences.dat");

}

int main(int argc, char *argv[]) {
  if(argc != 2) {
    cout << "Erreur !! la commande est de la forme << nisp degre >>" << endl;
    cout << "degre : degre du polynome" << endl;
  }
  else {
    int degre = atoi(argv[1]);
    GenererPlan(degre);
  }
}
