/*!
  \file   ValidationIshigami.C
  \brief  Validation du polynome
*/
#include <iostream>
#include <fstream>
#include <cmath>     // pour disposer de la macro M_PI = 3.14159...
#include "nisp_pc.h" // les classes Nisp

using namespace std;

// Fonction de Ishigami
// Modele y1(x1,x2,x3) = sin(x1) + a (sin(x2))**2 + b (x3)**4 sin(x1)
// Modele y2(x1,x2,x3) = 2 * [sin(x1) + a (sin(x2))**2 + b (x3)**4 sin(x1)]
void Ishigami(double x[], double y[]) {
  double u1,u2,u3,s1,s2,s3,a,b;
  a=7.; b=0.1;
  u1=x[0];    u2=x[1];    u3=x[2];
  s1=sin(u1); s2=sin(u2); s3=sin(u3);
  y[0] = s1 + a *s2*s2 + b * u3*u3*u3*u3 * s1;
  y[1]=2.*y[0];
}

/*!
  \fn void ValidationIshigami(int np)
  \brief Validation du PC
  \param np : nombre de simulations
*/
void ValidationIshigami(int np) {
  PolynomialChaos * pc = new PolynomialChaos((char *) "NispIshigami.dat");
  int nx=pc->GetDimensionInput();
  int ny=pc->GetDimensionOutput();

  RandomVariable * xu[nx] ;
  for(int i=0;i<nx;i++) xu[i] = new RandomVariable("Uniforme",-M_PI,+M_PI);
  
  RandomVariable * xi[nx] ;
  for(int i=0;i<nx;i++) xi[i] = new RandomVariable("Uniforme");

  // Initialisation de la "graine" aleatoire
  void InitSeed(int sp);
  InitSeed(0);

  ofstream sortie((char *) "validation.dat",ios::out);
  sortie.setf(ios::scientific, ios::floatfield);
  double input[nx], output[ny], inputpc[nx];
  for(int k=1;k<=np;k++) {
    for(int i=0;i<nx;i++) inputpc[i]=xi[i]->GetValue();
    for(int i=0;i<nx;i++) pc->SetInput(i+1,inputpc[i]);
    pc->ComputeOutput();

    for(int i=0;i<nx;i++) input[i]=xu[i]->GetValue(xi[i],inputpc[i]);
    Ishigami(input,output);
    for(int j=0;j<ny;j++) sortie << output[j] << " " << pc->GetOutput(j+1) << " ";
    sortie << endl;
  }
  sortie.close();
}

int main(int argc, char *argv[]) {
  if(argc != 2) {
    cout << "Erreur !! la commande est de la forme << nisp np >>" << endl;
    cout << "np : nombre de simulations" << endl;
  }
  else {
    int np = atoi(argv[1]);
    ValidationIshigami(np);
  }
}
