// 2006-2009 - CEA - Jean-Marc Martinez
// 2009 - INRIA - Michael Baudin
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <cmath>

#include <string.h>
#include <stdlib.h>


// Les classes Nisp
#include "nisp_pc.h"
#include "nisp_util.h"
#include "nisp_msg.h"

using namespace std;

#define HERMITE  1
#define LEGENDRE 2
#define LAGUERRE 3

// Coefficients du PolynomialChaos
void WriteCoef(PolynomialChaos * pc, ofstream & ficsrc, char *name) {
	int j,k,nx,no,ny, p;
	nx=pc->nx;
	no=pc->no;
	ny=pc->ny;
	p=pc->p;
	char racine[100]; 
	strcpy(racine,name); 
	strcat(racine,"_beta");
	ficsrc << "double "<<racine<<"["<<ny<<"][" <<p+1<< "]={"<<endl;
	int nl=0;
	for(j=1;j<=ny-1;j++) {
		for(k=0;k<=p;k++) {
			if(nl==8) {
				nl=0; 
				ficsrc<<endl;
			}
			nl++; 
			ficsrc << pc->beta[j][k] << ",";
		}
	}
	for(k=0;k<p;k++) {
		if(nl==8) {
			nl=0; 
			ficsrc<<endl;
		}
		nl++; 
		ficsrc << pc->beta[ny][k] << ",";
	}
	ficsrc << pc->beta[ny][p] << "};"<<endl;
}

// Tableau des multi indices
void WriteIndiceMultiple(PolynomialChaos * pc, ofstream & ficsrc, char *name) {
	int i,k,nx,no,ny, p;
	nx=pc->nx;
	no=pc->no;
	ny=pc->ny;
	p=pc->p;
	char racine[100]; 
	strcpy(racine,name); 
	strcat(racine,"_indmul");
	ficsrc << "int "<<racine<<"["<<p+1<<"][" <<nx<< "]={"<<endl;
	for(k=0;k<p;k++) {
		for(i=1;i<=nx;i++) {
			ficsrc << pc->indmul[k][i] <<",";
		}
		if(!(((k+1)*nx)%25)) {
			ficsrc<<endl;
		}
	}
	for(i=1;i<nx;i++) {
		ficsrc << pc->indmul[p][i] <<",";
	}
	ficsrc << pc->indmul[p][nx] <<"};"<<endl;
}

void WriteCodeLegendre(ofstream & ficsrc, char *name) {
	char racine[100]; 
	strcpy(racine,name); 
	strcat(racine,"_legendre");
	ficsrc<<"void "<<racine<<"(double *phi,double x, int no) {"<<endl;
	ficsrc<<"   int i;"<<endl;
	ficsrc<<"   x=2.*x-1.;"<<endl;
	ficsrc<<"   phi[0]=1.;"<<endl;
	ficsrc<<"   phi[1]=x;"<<endl;
	ficsrc<<"   for(i=1;i<no;i++) {"<<endl;
	ficsrc<<"	    phi[i+1]= ((2.*i+1.) * x * phi[i] - i * phi[i-1]) / (i+1.);"<<endl;
	ficsrc<<"   }"<<endl;
	ficsrc<<"   for(i=0;i<=no;i++) {"<<endl;
	ficsrc<<"	    phi[i]  = phi[i] * sqrt(2.* i + 1.);"<<endl;
	ficsrc<<"   }"<<endl;
	ficsrc<<"}"<<endl;
}

void WriteCodeHermite(ofstream & ficsrc, char *name) {
	char racine[100]; 
	strcpy(racine,name); 
	strcat(racine,"_hermite");
	ficsrc<<"void "<<racine<<"(double *phi,double x, int no) {"<<endl;
	ficsrc<<"   int i,j;"<<endl;
	ficsrc<<"   double c1,c2;"<<endl;
	ficsrc<<"   x=x/sqrt(2.);"<<endl;
	ficsrc<<"   phi[0]=1.;"<<endl;
	ficsrc<<"   phi[1]=2*x;"<<endl;
	ficsrc<<"   for(i=1;i<no;i++) {"<<endl;
	ficsrc<<"	    phi[i+1]= 2. * x * phi[i] - 2. * i * phi[i-1];"<<endl;
	ficsrc<<"   }"<<endl;
	ficsrc<<"   for(i=0;i<=no;i++) {"<<endl;
	ficsrc<<"      c1=pow(2.,(double) i);"<<endl;
	ficsrc<<"      for(c2=1.,j=1;j<=i;j++) {"<<endl;
	ficsrc<<"        c2=c2*j;"<<endl;
	ficsrc<<"      }"<<endl;
	ficsrc<<"      phi[i]  = phi[i] / sqrt(c1*c2);"<<endl;
	ficsrc<<"   }"<<endl;
	ficsrc<<"}"<<endl;
}
void WriteCodeLaguerre(ofstream & ficsrc, char *name) {
	char racine[100]; 
	strcpy(racine,name); 
	strcat(racine,"_laguerre");
	ficsrc<<"void "<<racine<<"(double *phi,double x, int no) {"<<endl;
	ficsrc<<"   int i;"<<endl;
	ficsrc<<"   phi[0]=1.;"<<endl;
	ficsrc<<"   phi[1]=1.-x;"<<endl;
	ficsrc<<"   for(i=1;i<no;i++) {"<<endl;
	ficsrc<<"	    phi[i+1]= ((2.*i+1.-x) * phi[i] - i * phi[i-1]) / (i+1.);"<<endl;
	ficsrc<<"   }"<<endl;
	ficsrc<<"}"<<endl;
}

// Polynomes orthogonaux
void WriteCodeSrc(PolynomialChaos * pc, ofstream & ficsrc, char *name) {
	int i,nx,no,ny, p;
	nx=pc->nx;
	no=pc->no;
	ny=pc->ny;
	p=pc->p;
	ficsrc <<"   int i,j,k,nx,ny,no,p;"<<endl;
	ficsrc <<"   double psi["<<p+1<<"],phi["<<nx<<"]["<<no+1<<"],xi["<<nx<<"],s;"<<endl;
	ficsrc <<"   nx="<<nx<<";"<<endl;
	ficsrc <<"   ny="<<ny<<";"<<endl;
	ficsrc <<"   no="<<no<<";"<<endl;
	ficsrc <<"   p="<<p<<";"<<endl;
	ficsrc <<"   for(i=0;i<nx;i++) {"<<endl;
	ficsrc <<"	    xi[i]=x[i];"<<endl;
	ficsrc <<"   }"<<endl;
	char racine[100];
	for(i=1;i<=nx;i++) {
		if(pc->funpol[i-1]==HERMITE) {
			strcpy(racine,name); 
			strcat(racine,"_hermite");
			ficsrc<<"   "<<racine<<"(phi["<<i-1<<"],xi["<<i-1<<"],"<<no<<");"<<endl;
		}
		else if (pc->funpol[i-1]==LEGENDRE){
			strcpy(racine,name); 
			strcat(racine,"_legendre");
			ficsrc<<"   "<<racine<<"(phi["<<i-1<<"],xi["<<i-1<<"],"<<no<<");"<<endl;
		}
		else {
			strcpy(racine,name); 
			strcat(racine,"_laguerre");
			ficsrc<<"   "<<racine<<"(phi["<<i-1<<"],xi["<<i-1<<"],"<<no<<");"<<endl;
		}
	}
	strcpy(racine,name); 
	strcat(racine,"_indmul");
	ficsrc<<"   for(k=0;k<=p;k++) {"<<endl;
	ficsrc<<"      for(psi[k]=1.,i=0;i<nx;i++) {"<<endl;
	ficsrc<<"		   psi[k]=psi[k]*phi[i]["<<racine<<"[k][i]];"<<endl;
	ficsrc<<"      }"<<endl;
	ficsrc<<"   }"<<endl;

	strcpy(racine,name); 
	strcat(racine,"_beta");
	ficsrc<<"   for(j=0;j<ny;j++) {"<<endl;
	ficsrc<<"      for(s=0.,k=0;k<=p;k++) {"<<endl;
	ficsrc<<"		   s+="<<racine<<"[j][k]*psi[k];"<<endl;
	ficsrc<<"      }"<<endl;
	ficsrc<<"      y[j]=s;"<<endl;
	ficsrc<<"   }"<<endl;

}
// Generation du code source du PolynomialChaos
void PolynomialChaos::GenerateCode(char *pcsrc, char *name) {

	ofstream ficsrc(pcsrc,ios::out);
	if(!ficsrc) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(PolynomialChaos::GenerateCode) : Problem when open the file " << pcsrc << endl;
		nisp_error ( msg.str() );
		return;
	}
	ficsrc<<"#include <math.h>"<<endl<<endl;

	WriteCoef(this,ficsrc,name);
	WriteIndiceMultiple(this,ficsrc,name);

	int flaghermite=0;
	int flaglegendre=0;
	int flaglaguerre=0;
	for(int i=1;i<=nx;i++) {
		if(funpol[i-1]==HERMITE)         {
			if(!flaghermite)  {
				WriteCodeHermite(ficsrc,name); 
				flaghermite =1;
			}
		}
		else if (funpol[i-1]==LEGENDRE)  {
			if(!flaglegendre) {
				WriteCodeLegendre(ficsrc,name);
				flaglegendre=1;
			}
		}
		else if (funpol[i-1]==LEGENDRE)  {
			if(!flaglaguerre) {
				WriteCodeLaguerre(ficsrc,name);
				flaglaguerre=1;
			}
		}
	}


	ficsrc << "void "<<name<<"(double *x, double *y) {" << endl;
	WriteCodeSrc(this,ficsrc,name);
	ficsrc << "}"<<endl;
	ficsrc.close();
}


