// 2006-2009 - CEA - Jean-Marc Martinez
// 2009 - INRIA - Michael Baudin
// 2009 - Digiteo - Michael Baudin
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <vector>

#include <stdlib.h>
#include <exception>

#include "nisp_gva.h"
#include "nisp_util.h"
#include "nisp_math.h"
#include "nisp_msg.h"
#include "nisp_smolyak.h"

using namespace std;


// ===================
// Methodes classe SetRandomVariable
// ===================
SetRandomVariable::SetRandomVariable() {
	nx=0;
	np=0;
	datafile_nbchar = 17;
	datafile_precision = 8;
}
SetRandomVariable::~SetRandomVariable() {
	// TODO : destroy the included random variables if they were automatically created with a read in the data file
	this->FreeMemory();
}


SetRandomVariable::SetRandomVariable(int n) { // par defaut les variables sont uniformes et normalisees 
	nx=n;
	np=0;
	datafile_nbchar = 17;
	datafile_precision = 8;
	for(int j=1;j<=n;j++) {
		this->AddRandomVariable(new RandomVariable("Uniforme"));
	}
}
SetRandomVariable::SetRandomVariable(char *fichier) {
	datafile_nbchar = 17;
	datafile_precision = 8;
	ifstream entree(fichier,ios::in);
	if(!entree) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetRandomVariable) : problem to open the file " << fichier << endl;
		nisp_error ( msg.str() );
		return;
	}
	string ttypesampling,type,str; 
	int nnp,nnx,dmax; 
	double a,b;
	entree >> str >> str >> str >> ttypesampling;
	entree >> str >> str >> str >> nnp;
	entree >> str >> str >> str >> nnx;
	entree >> str >> str >> str >> dmax;
	for(int i=1;i<=nnx;i++) {
		entree >> str >> str >> str >> type >> a >> b;
		this->AddRandomVariable(new RandomVariable(type,a,b));
	}
	if(nnp) {
		x=dmatrix(nnp+1,nx+1); 
		w=dvector(nnp+1);
		for(int k=1;k<=nnp;k++) {
			for(int i=1;i<=nnx;i++) {
				entree >> x[k][i]; 
			}
			entree >> w[k];
		}
	}
	entree.close();
	nx=nnx; 
	np=nnp; 
	typesampling=ttypesampling; 
	degre_max=dmax;
}

void SetRandomVariable::FreeMemory() {
	if(np) {
		free_dmatrix(x,np+1);
		free_dvector(w);
		np=0;
	}
}
void SetRandomVariable::AddRandomVariable(RandomVariable *x) {
	va.push_back(x); 
	nx=(int) va.size();
}
void SetRandomVariable::BuildSample(char * name, int n) {
	string sname;
	sname.assign(name);
	this->BuildSample ( sname , n );
}
void SetRandomVariable::BuildSample(string name, int n) {
	typesampling=name;
	if(n<0) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : level or size " << n << " < 0" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(nx<1) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : stochastic dimension = " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(np) {
		FreeMemory();
	}
	if     (typesampling=="MonteCarlo") {
		nisp_montecarlo(this,n);
	}
	else if(typesampling=="Lhs")        {
		nisp_lhs(this,n);
	}
	else if(typesampling=="QmcSobol")   {
		nisp_qmcsobol(this,n);
	}
	else if(typesampling=="Quadrature") {
		nisp_quadrature(this,n);
	}
	else if(typesampling=="Petras")     {
		nisp_petras(this,n);
	}
	else if(typesampling=="SmolyakGauss")   {
		smolyak(this,typesampling,n); // JMModif 26 mai 2009
	}
	else if(typesampling=="SmolyakTrapeze") {
		smolyak(this,typesampling,n); // JMModif 26 mai 2009
	}
	else if(typesampling=="SmolyakFejer")   {
		smolyak(this,typesampling,n); // JMModif 26 mai 2009
	}
	else if(typesampling=="SmolyakClenshawCurtis")   {
		smolyak(this,typesampling,n); // JMModif 28 mai 2009
	}
	else {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : the method " << name << " is unknown" << endl;
		nisp_error ( msg.str() );
		return;
	}
}

// TODO : what is "name" for if it is not used ?
void SetRandomVariable::SetSample(char * name, int n) {
	string sname;
	sname.assign(name);
	this->SetSample ( sname , n );
}
void SetRandomVariable::SetSample(string name, int n) {
	if(n<1) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : size " << n << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	degre_max=-1;
	if(nx<1) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : stochastic dimension = " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(np!=0) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the current number of simulations is np=" << np << endl;
		msg << "Call FreeMemory before configuring a new sampling." << endl;
		nisp_error ( msg.str() );
		return;
	}
	np=n; 
	typesampling = name;
	x=dmatrix(np+1,nx+1); 
	w=dvector(np+1);
	for(int k=1;k<=np;k++) {
		for(int i=1;i<=nx;i++) {
			x[k][i] = 0.0;
		}
		w[k] = 0.0;
	}
}

void SetRandomVariable::SetSample(int k, int i, double value) {
	if(k<1)  {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the index of sample k= " << k << " is < 1 " <<endl;
		nisp_error ( msg.str() );
		return;
	}
	if(k>np) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the index of sample k= " << k << " is > " << np << " = size of sampling" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(i<1)  {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the index of variable i= " << i << " is < 1 " <<endl;
		nisp_error ( msg.str() );
		return;
	}
	if(i>nx) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the index of variable i= " << i << " is > stochastic dimension = " << nx << endl;
		nisp_error ( msg.str() );
		return;
	}
	x[k][i]=value;
}

void SetRandomVariable::SetSample(int k, double * value) {
	if(k<1)  {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the index of sample k= " << k << " is < 1 " <<endl;
		nisp_error ( msg.str() );
		return;
	}
	if(k>np) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::SetSample) : the index of sample k= " << k << " is > " << np << " = size of sampling" << endl;
		nisp_error ( msg.str() );
		return;
	}
	
	// TODO : use BLAS to make copy
	for(int i=1;i<=nx;i++) {
		x[k][i] = value[i-1];
	}
}

void SetRandomVariable::SetSample ( double ** value ) {
	// TODO : use BLAS to make copy
	for(int k=1;k<=np;k++) {
		for(int i=1;i<=nx;i++) {
			x[k][i] = value[k-1][i-1];
		}
	}
}

void SetRandomVariable::BuildSample(char * name, int n, int ne) {
	string sname;
	sname.assign(name);
	this->BuildSample ( sname , n , ne );
}
void SetRandomVariable::BuildSample(string name, int n, int ne) {
	if(n<1) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : size " << n << " < 1" <<endl;
		nisp_error ( msg.str() );
		return;
	} 
	if(ne<1) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : number of Lhs for LhsMaxMin = " << ne << " < 1"<<endl;
		nisp_error ( msg.str() );
		return;
	} 
	if(nx<1) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : stochastique dimension = " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	typesampling=name;
	if(typesampling=="LhsMaxMin")  {
		if(np) {
			FreeMemory();
		}
		nisp_lhsmaxmin(this,n,ne);
	}
	else {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : method " << name << " different of LhsMaxMin" << endl;
		nisp_error ( msg.str() );
		return;
	}
}

void SetRandomVariable::BuildSample(SetRandomVariable *mother) {
	if(nx != mother->nx) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(setRandomVariable::BuildSample) : stochastic dimension are not equal between the two set of random variables" << endl;
		msg << "The child is " << nx << "while the mother is " << mother->nx << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(mother->np==0) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::BuildSample) : the size of the mother set is empty" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(np) {
		FreeMemory();
	}
	typesampling=mother->typesampling;
	np=mother->np;
	degre_max=mother->degre_max;
	// TODO : a matrix with (np) x (nx) elements should be sufficient, provided that k=0,...,np-1 and i=0,...,nx-1
	x=dmatrix(np+1,nx+1); 
	w=dvector(np+1);
	for(int k=1;k<=np;k++) {
		for(int i=1;i<=nx;i++) {
			x[k][i]= va[i-1]->GetValue(mother->va[i-1],mother->x[k][i]);
		}
	}
	for(int k=1;k<=np;k++) {
		w[k]=mother->w[k];
	}
}

void SetRandomVariable::Save(char *fichier) {
	ofstream sortie(fichier,ios::out);
	if(!sortie) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::Save) : Problem when open file " << fichier << endl;
		nisp_error ( msg.str() );
		return;
	}
	sortie << "# Type         : " << typesampling << endl;
	sortie << "# Simulations  : " << np << endl;
	sortie << "# Variables    : " << nx << endl;
	sortie << "# DegreMax     : " << degre_max << endl;
	for(int i=0;i<nx;i++) {
		sortie << "# PDF          : " << va[i]->type << " " << va[i]->a << " " << va[i]->b << endl;
	}
	sortie << endl;
	sortie.setf(ios::scientific, ios::floatfield);
	sortie.precision(datafile_precision);
	for(int k=1;k<=np;k++) {
		for(int i=1;i<=nx;i++) { 
			sortie.width(datafile_nbchar); 
			sortie << x[k][i]; 
		}
		sortie.width(datafile_nbchar); 
		sortie << w[k];
		sortie << endl;
	}
	sortie.close();
}

int SetRandomVariable::GetDimension() {
	return(nx);
}

int SetRandomVariable::GetSize() {
	return(np);
}

void SetRandomVariable::GetLog() {
	for(int i=0;i<nx;i++) {
		va[i]->GetLog();
	}
}

double SetRandomVariable::GetSample(int k, int i) {
	double result = 0.0;
	if (np==0) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : There is no sample in database ; np= " << np << endl;
		nisp_error ( msg.str() );
		return result;
	}
	if(k<1)  {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : the index of sample k= " << k << " is < 1 " <<endl;
		nisp_error ( msg.str() );
		return result;
	}
	if (k>np) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : The index k=" << k << "is larger than the number of samples np= " << np << endl;
		nisp_error ( msg.str() );
		return result;
	}
	if(i<1)  {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : the index of variable i= " << i << " is < 1 " <<endl;
		nisp_error ( msg.str() );
		return result;
	}
	if(i>nx) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : the index of variable i= " << i << " is > stochastic dimension = " << nx << endl;
		nisp_error ( msg.str() );
		return result;
	}
	result = x[k][i];
	return result;
}

void SetRandomVariable::GetSample( int k , double * sample ) {
	if (np==0) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : There is no sample in database - np= " << np << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(k<1)  {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : the index of sample k= " << k << " is < 1 " <<endl;
		nisp_error ( msg.str() );
		return;
	}
	if (k>np) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : The index k=" << k << "is larger than the number of samples np= " << np << endl;
		nisp_error ( msg.str() );
		return;
	}
	// TODO : use BLAS to make copy ?
	for(int i=1;i<=nx;i++) {
		sample[i-1] = x[k][i];
	}
}

void SetRandomVariable::GetSample( double ** sample ) {
	if (np==0) {
		ostringstream msg;
		msg << "NISP - ERROR" << endl;
		msg << "Nisp(SetRandomVariable::GetSample) : There is no sample in database - np= " << np << endl;
		nisp_error ( msg.str() );
		return ;
	}
	// TODO : use BLAS to make copy
	for(int k=1;k<=np;k++) {
		for(int i=1;i<=nx;i++) {
			sample[k-1][i-1] = x[k][i];
		}
	}
}

