// 2006-2009 - CEA - Jean-Marc Martinez
// 2009 - Digiteo - Michael Baudin
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <vector>
#include <math.h>
#include <algorithm>

#include "calerf.h"
#include "linpack_d.h"
#include "nisp_gva.h"
#include "nisp_inv.h"
#include "nisp_math.h"
#include "nisp_msg.h"
#include "nisp_pc.h"
#include "nisp_qua.h"
#include "nisp_random.h"
#include "nisp_smo.h"
#include "nisp_util.h"
#include "sobol.h"

using namespace std;

// PDF
double nisp_normale(double mean, double std) {
	return(mean + std * nisp_randnor ());
}
double nisp_lognormale(double mean, double std) {
	double mu=log(mean)-std*std/2.;
	return(exp(nisp_normale(mu,std))); // mu = moyenne de la nisp_lognormale mais std = sigma de la nisp_normale
}
double nisp_uniforme(double min, double max) {
	return(min + (max-min) * nisp_randuni ());
}
double nisp_loguniforme(double min, double max) {
	return(exp(nisp_uniforme(log(min),log(max)))); // min et max de la variable
}
double nisp_exponentielle(double lambda, double null) {
	return( nisp_randexp ()/lambda);
}
// mere est uniforme dans [0,1]
double nisp_pdfChange ( RandomVariable *fils, double z) {
	double y;
	double a=fils->a; 
	double b=fils->b;
	string typefils=fils->type;
	if((z<0.) || (z>1.)) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_pdfChange) : value " << z << "is not in [0,1]" << endl;
		nisp_error ( msg.str() );
		return(0.);
	}
	if(typefils=="Uniforme") {
		y=a+(b-a)*z;
	}
	else if(typefils=="Normale") {
		y=a+b*nisp_invfcn(z);
	}
	else if(typefils=="Exponentielle") {
		y=(-1./a)*log(z);
	}
	else if(typefils=="LogNormale") {
		y=exp(log(a)-b*b/2.+b*nisp_invfcn(z));
	}
	else if(typefils=="LogUniforme") {
		y=exp(log(a)+log(b/a)*z);
	}
	else 
	{
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_pdfChange) : law " << typefils << " unknown" << endl;
		nisp_error ( msg.str() );
		return(0.);
	}
	return(y);
}
double nisp_pdfChange ( RandomVariable *fils, RandomVariable *mere, double x) {
	double y;
	double a=fils->a; 
	double b=fils->b;
	string typefils=fils->type; 
	string typemere=mere->type;
	ostringstream msg;
	if(typemere=="Normale") 
	{            // Mere est Normale
		double mean=mere->a; 
		double sigma=mere->b;
		double z=(x-mean)/sigma;           // z suit une loi normale centree reduite
		if(typefils=="Normale") {
			y=a+b*z;
		}
		else if(typefils=="Uniforme") {
			y=a+(b-a)*(calerfc(-z/sqrt(2.))/2.);
		}
		else if(typefils=="Exponentielle") {
			y=-(1./a)*log((calerfc(-z/sqrt(2.))/2.));
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*z);
		}
		else if(typefils=="LogUniforme") {
			y=exp(log(a)+log(b/a)*(calerfc(-z/sqrt(2.))/2.));
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : Transformation Normale -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="LogNormale") 
	{    // Mere est LogNormale
		if(x<=0.) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : law (mother)is LogNormale and value " << x << " < 0" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double mean=mere->a; 
		double sigma=mere->b; 
		double mu=log(mean)-sigma*sigma/2.;
		double z=(log(x)-mu)/sigma;        // z suit une loi normale centree reduite
		if(typefils=="Normale") {
			y=a+b*z;
		}
		else if(typefils=="Uniforme") {
			y=a+(b-a)*(calerfc(-z/sqrt(2.))/2.);
		}
		else if(typefils=="Exponentielle") {
			y=-(1./a)*log((calerfc(-z/sqrt(2.))/2.));
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*z);
		}
		else if(typefils=="LogUniforme") {
			y=exp(log(a)+log(b/a)*(calerfc(-z/sqrt(2.))/2.));
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : Transformation LogNormale -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="Uniforme") 
	{      // Mere est Uniforme
		double min=mere->a; 
		double max=mere->b;
		if((x<min) || (x>max)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : value " << x << " must be in ["<<min<<","<<max<<"]"<< endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double z=(x-min)/(max-min);        // z suit une loi uniforme [0,1]
		if(typefils=="Uniforme") {
			y=a+(b-a)*z;
		}
		else if(typefils=="Normale") {
			y=a+b*nisp_invfcn(z);
		}
		else if(typefils=="Exponentielle") {
			y=(-1./a)*log(z);
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*nisp_invfcn(z));
		}
		else if(typefils=="LogUniforme") {
			y=exp(log(a)+log(b/a)*z);
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : Transformation Uniform -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="LogUniforme") 
	{                // Mere est LogUniforme
		double min=mere->a; 
		double max=mere->b;
		if(x<=0.) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : law (mother)is LogUniform and value " << x << " < 0" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double z=(log(x)-log(min))/(log(max)-log(min)); // z suit une loi uniforme [0,1]
		if((z<0.) || (z>1.)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : law (mother) is LogUniform and error on value " << x << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		if(typefils=="Uniforme") {
			y=a+(b-a)*z;
		}
		else if(typefils=="Normale") {
			y=a+b*nisp_invfcn(z);
		}
		else if(typefils=="Exponentielle") {
			y=(-1./a)*log(z);
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*nisp_invfcn(z));
		}
		else if(typefils=="LogUniforme") {
			y=exp(log(a)+log(b/a)*z);
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : Transformation LogUniform -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="Exponentielle") 
	{ // Mere est Exponentielle
		double lambda=mere->a;
		if(x<=0.) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : law (mother)is Exponential and value " << x << " <= 0" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double z=x/lambda;                 // z suit une loi exponentielle lambda=1;
		if(typefils=="Exponentielle") {
			y=z/a;
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(nisp_pdfChange) : Transformation Exponential -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else 
	{
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_pdfChange) : law " << typemere << " unknown" << endl;
		nisp_error ( msg.str() );
		return(0.);
	}
	return(y);
}
// Monte Carlo
void nisp_montecarlo(SetRandomVariable *gva, int np) {
	gva->degre_max=-1;
	int nx=gva->nx;
	if(nx<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_montecarlo) : stochastic dimension " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	gva->np=np; 
	gva->x=dmatrix(np+1,nx+1); 
	gva->w=dvector(np+1);
	int k,j; 
	double pas=1./(double) np;
	for(k=1;k<=np;k++) {
		for(j=1;j<=nx;j++) {
			gva->x[k][j]=gva->va[j-1]->GetValue();
		}
	}
	for(k=1;k<=np;k++) {
		gva->w[k]= pas;
	}
}
// Latin Hypercube Sampling
void nisp_lhs(SetRandomVariable *gva, int np) {
	gva->degre_max=-1;
	int nx=gva->nx;
	if(nx<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_lhs) : stochastic dimension " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	gva->np=np;
	gva->x=dmatrix(np+1,nx+1); 
	gva->w=dvector(np+1);
	int k,j; 
	double pas=1./(double) np;
	vector<double> v(np);
	for(k=0;k<np;k++) {
		v[k]=(0.5 + (double) k)*pas;
	}
	for(j=1;j<=nx;j++) {
		random_shuffle(v.begin(),v.end());
		for(k=1;k<=np;k++) {
			gva->x[k][j]=nisp_pdfChange(gva->va[j-1],v[k-1]);
		}
	}
	for(k=1;k<=np;k++) {
		gva->w[k]= pas;
	}
}
// Quasi Monte Carlo Sobol sequence (Numerical Recipies)
void nisp_qmcsobol(SetRandomVariable *gva, int np) {
	gva->degre_max=-1;
	int nx=gva->nx;
	if(nx<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_qmcsobol) : stochastic dimension " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	gva->np=np; 
	gva->x=dmatrix(np+1,nx+1); 
	gva->w=dvector(np+1);

	long long int seed=0;

	double * x = dvector(nx);
	double pas=1./(double) np;
	int k,j;
	i8_sobol(nx, &seed, x); // on passe le premier point qui est toujours x[i]=0 i=0,1,...,nx-1
	for(k=1;k<=np;k++) {
		i8_sobol(nx, &seed, x);
		for(j=1;j<=nx;j++) {
			gva->x[k][j]=nisp_pdfChange(gva->va[j-1],x[j-1]);
		}
		gva->w[k]=pas;
	}
	free_dvector(x);
}
// Latin Hypercube Sampling critere Max Min
void nisp_lhsmaxmin(SetRandomVariable *gva, int np, int ne) {
	int i,j,k,ii,kk,k1;
	double **xt,dmin,dmaxmin,d,dx;
	int nx=gva->nx;
	if(nx<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_lhsmaxmin) : stochastic dimension " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	gva->np=np; 
	gva->x=dmatrix(np+1,nx+1); 
	gva->w=dvector(np+1);
	gva->degre_max=-1;
	vector<double> v(np); 
	double pas=1./((double) np);
	for(k=0;k<np;k++) {
		v[k]=(0.5 + (double) k)*pas;
	}
	dmaxmin=0.;
	xt=dmatrix(np+1,nx+1);
	for(int b=1;b<=ne;b++) {
		for(j=1;j<=nx;j++) {
			random_shuffle(v.begin(),v.end());
			for(k=1;k<=np;k++) {
				xt[k][j]=v[k-1];
			}
		}
		// MaxMin
		dmin=1.e45;
		for(k=1;k<=np;k++) {
			for(k1=k+1;k1<=np;k1++) {
				for(d=0.,i=1;i<=nx;i++) { 
					dx=xt[k][i]-xt[k1][i]; 
					d+=dx*dx;
				}
				if(d<dmin) {
					dmin=d;
				}
			}
		}
		if(dmaxmin<dmin) { // Sauvegarde du plan Maximisant la plus petite distance entre 2 points
			dmaxmin=dmin;
			for(ii=1;ii<=nx;ii++) {
				for(kk=1;kk<=np;kk++) {
					gva->x[kk][ii]=xt[kk][ii];
				}
			}
		}
	}
	// Changements de variable
	for(i=1;i<=nx;i++) {
		for(k=1;k<=np;k++) {
			gva->x[k][i]=nisp_pdfChange(gva->va[i-1],gva->x[k][i]);
		}
	}
	free_dmatrix(xt,np+1);
	for(k=1;k<=np;k++) {
		gva->w[k]=pas;
	}
}
// Test des variables stochastiques
void teststo(string type,double a, double b) {
	int test=0;
	if(type=="Normale")            { 
		if((a==0.)&&(b=1.)) {
			test=1;
		}
	}
	else if(type=="Uniforme")      { 
		if((a==0.)&&(b=1.)) {
			test=1;
		}
	}
	else if(type=="Exponentielle") { 
		if((a==1.)&&(b=1.)) {
			test=1;
		}
	}
	if(!test) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(teststo) : variables are not stochastic " << type << " " << a << " " << b << endl;
		nisp_error ( msg.str() );
	}
}
// Test des variables stochastiques uniformes
void testnor(string type,double a, double b) {
	int test=0;
	if(type=="Uniforme")            { 
		if((a==0.)&&(b=1.)) {
			test=1;
		}
	}
	if(!test) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(teststo) : variables are not stochastic-uniform " << type << " " << a << " " << b << endl;
		nisp_error ( msg.str() );
	}
}
// Quadrature Tensorisee
void nisp_quadrature(SetRandomVariable *gva, int degre) {
	int nx=gva->nx;
	int nq=degre+1; // le nombre de points doit etre egal au degre + 1

	if(nx<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_quadrature) : stochastic dimension = " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(degre<0) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_quadrature) : degree of quadrature == " << degre << " < 0" << endl;
		nisp_error ( msg.str() );
		return;
	}
	// degr?
	gva->degre_max=degre;
	// On verifie que les variables aleatoires sont normalisees
	int j,np;
	for(j=1;j<=nx;j++) {
		teststo(gva->va[j-1]->type,gva->va[j-1]->a,gva->va[j-1]->b);
	}
	for(np=1,j=1;j<=nx;j++) np=np*nq;
	gva->np=np;
	gva->x=dmatrix(np+1,nx+1); gva->w=dvector(np+1);
	// Calcul la quadrature tensorisee
	double **xqt, **wqt;
	xqt=dmatrix(nx+1,nq+1); 
	wqt=dmatrix(nx+1,nq+1);
	for(j=1;j<=nx;j++) {
		Quadrature(xqt[j],wqt[j],nq,gva->va[j-1]->type);
	}
	QuadratureTensorise(gva->x,gva->w,xqt,wqt,nx,nq,np);
	free_dmatrix(xqt,nx+1); free_dmatrix(wqt,nx+1);
}
// Quadrature Tensorisee Incomplete (Smolyak, Petras)
// Le niveau correspond au degre max des PC qu'on peut calculer
void nisp_petras(SetRandomVariable *gva, int degre) {
	int niveau=degre;
	if(degre<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_petras) : degree of cubature == " << degre << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(niveau>40) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_petras) : level of cubature == " << niveau << " > 40 (value max of the level)" << endl;
		nisp_error ( msg.str() );
		return;
	}
	int k,icpt,i,j,nnx,q,nquad;
	double *ps,*ws;
	int nx=gva->nx;
	if(nx<1) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_petras) : stochastic dimension = " << nx << " < 1" << endl;
		nisp_error ( msg.str() );
		return;
	}
	// Degre maximum
	gva->degre_max=degre;
	// On verifie que les variables aleatoires sont normalisees
	for(j=1;j<=nx;j++) {
		teststo(gva->va[j-1]->type,gva->va[j-1]->a,gva->va[j-1]->b);
	}
	nnx=nx;
	q=niveau+nnx;
	size_smolyak(&nnx,&q,&nquad);
	ws=dvector(nquad+1);
	ps=dvector(nquad*nnx + 1);
	quad_smolyak(&nx,&q,ps,ws);
	gva->np=nquad;
	gva->x=dmatrix(nquad+1,nx+1);
	gva->w=dvector(nquad+1);
	for(k=1,icpt=0,i=0;i<nquad;i++) {
		for(j=1;j<=gva->nx;j++) {
			gva->x[k][j]=nisp_pdfChange(gva->va[j-1],ps[icpt++]);
		}
		gva->w[k]=ws[i];
		k++;
	}
	free_dvector(ws);
	free_dvector(ps);
}
// Calcul des coefficients par integration (quadrature de Gauss)
void CoefficientsIntegration(double **beta, double **Z, double *w, double **y, int p, int np, int ny) {
	int i,j,k; double s;
	for(j=1;j<=ny;j++) {
		for(k=0;k<=p;k++) {
			for(s=0.,i=1;i<=np;i++) {
				s+=Z[i][k]*w[i]*y[i][j]; 
			}
			beta[j][k]=s;
		}
	}
}

void nisp_coefficientsregression(double **beta, double *Z, double **y, int p, int np, int ny) {
	int i,j,k;

	int m=np;
	int n=p+1;
	int lda=m;
	int ldv=n;
	int ldu=m;
	int job = 21;
	double * s = dvector(m+n);
	double * e = dvector(m+n);
	double * u = dvector(m*n);
	double * v = dvector(n*n);
	double * work = dvector(m);
	double * tmp = dvector(n);

	dsvdc(Z,lda,m,n,s,e,u,ldu,v,ldv,work,job);

	// Transformation des vecteurs de la matrice V (regularisation)
	double coef, alpha=1.e-6;
	for(j=1;j<=n;j++) {
		coef=0.; 
		if(s[j-1]) {
			coef=s[j-1]/(s[j-1]*s[j-1]+alpha);
		}
		for(i=1;i<=n;i++) {
			v[(i-1)+(j-1)*n]=coef*v[(i-1)+(j-1)*n];
		}
	}
	double st;
	for(j=1;j<=ny;j++) {
		for(i=1;i<=n;i++) {
			for(st=0.,k=1;k<=m;k++) {
				st+=u[(k-1)+(i-1)*m]*y[k][j];  
			}
			tmp[i-1]=st;
		}
		for(k=1;k<=n;k++) {
			for(st=0.,i=1;i<=n;i++) {
				st+=v[(k-1)+(i-1)*n]*tmp[i-1]; 
			}
			beta[j][k-1]=st;
		}
	}
	free_dvector(s);
	free_dvector(e);
	free_dvector(u);
	free_dvector(v);
	free_dvector(work);
	free_dvector(tmp);
}

// Utiliser par la methode pc::sobol()
int nisp_puissance2(int n) {
	int i,ip;
	for(ip=1,i=1;i<=n;i++) {
		ip=ip*2;
	}
	return(ip);
}
// Lecture des simulations
void nisp_readtarget2(double **y, int np, int ny, char *fichier) {
	int k,j,nlignes,ncolonnes;
	ifstream entree(fichier,ios::in);
	if(!entree) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_readtarget2) : problem to open the file " << fichier << endl;
		nisp_error ( msg.str() );
		return;
	}
	string str;
	entree >> str >> nlignes;
	entree >> str >> ncolonnes;
	if(nlignes   != np) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_readtarget2) : number of simulations " << nlignes << " != " << np << endl;
		nisp_error ( msg.str() );
		return;
	}
	if(ncolonnes != ny) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(nisp_readtarget2) : number of output " << ncolonnes << " != " << ny << endl;
		nisp_error ( msg.str() );
		return;
	}
	for(k=1;k<=nlignes;k++) {
		for(j=1;j<=ncolonnes;j++) {
			entree >> y[k][j];
		}
	}
	entree.close();
}
// Utilitaires
int nisp_calculP(int nx, int no) {
	long min, max, num, den, i, p;
	if(nx>no) {
		min=(long) no; 
		max=(long) nx;
	}
	else      {
		min=(long) nx; 
		max=(long) no;
	}
	for(num=1,i=max+1;i<=max+min;i++) {
		num*=i;
	}
	for(den=1,i=2;i<=min;i++) {
		den*=i;
	}
	p=num/den-1;
	return((int) p);
}

// Probleme grandes valeurs
double binomial(int n, int j) {
	int k;
	double num,den;
	for(num=1.,k=n-j+1;k<=n;k++) {
		num=num*k;
	}
	for(den=1.,k=1;k<=j;k++)     {
		den=den*k;
	}
	return(num/den);
}


