// 2006-2009 - CEA - Jean-Marc Martinez
// 2009 - INRIA - Michael Baudin
// 2009 - Digiteo - Michael Baudin
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <vector>

#include "nisp_va.h"
#include "nisp_util.h"
#include "nisp_msg.h"

using namespace std;

// ===========
// Methodes RandomVariable
// ===========
void RandomVariable::SetDefaultParameters() {
	if     (type=="Normale")       {
		a=0.;  
		b=1.;
	}
	else if(type=="Uniforme")      {
		a=0.;  
		b=1.;
	}
	else if(type=="Exponentielle") {
		a=1.;  
		b=0.; 
	}
	else if(type=="LogNormale")    {
		a=0.1; 
		b=1.;
	}
	else if(type=="LogUniforme")   {
		a=0.1; 
		b=1.;
	}
	else  {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " unknown" << endl;
		nisp_error ( msg.str() );
	}
}
void RandomVariable::AssignFunction() {
	ostringstream msg;
	if (type=="Normale") {
		function = nisp_normale;
	}
	else if(type=="Uniforme") {
		function = nisp_uniforme;
	}
	else if(type=="LogNormale") {
		function = nisp_lognormale;
	}
	else if(type=="LogUniforme")   {
		function = nisp_loguniforme;
	}
	else if (type=="Exponentielle") {
		function = nisp_exponentielle;
	}
	else {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(RandomVariable::RandomVariable) : unknown law " << type << " with 2 parameters" << endl;
		nisp_error ( msg.str() );
	}
}
void RandomVariable::CheckParameters() {
	ostringstream msg;
	if (type=="Normale") {
		if(b<0) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and standard error " << b << " < 0 " << endl;
			nisp_error ( msg.str() );
		}
	}
	else if(type=="Uniforme") {
		if(a>=b) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and min = " << a << " >= max = " << b << endl;
			nisp_error ( msg.str() );
		}
	}
	else if(type=="LogNormale") {
		if((a<=0.) || (b<=0.)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and one of parameters are " << a << " , " << b << " <=0" << endl;
			nisp_error ( msg.str() );
		}
	}
	else if(type=="LogUniforme")   {
		if((a<=0.) || (b<=0.)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and one of parameters are " << a << " , " << b << " <=0" << endl;
			nisp_error ( msg.str() );
		}
		if(a>b) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and min = " << a << " > max = " << b << endl;
			nisp_error ( msg.str() );
		}
	}
	else if (type=="Exponentielle") {
		// Nothing to do
	}
	else {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(RandomVariable::RandomVariable) : unknown law " << type << " with 2 parameters" << endl;
		nisp_error ( msg.str() );
	}
}
RandomVariable::RandomVariable(string name) {
	type=name;
	SetDefaultParameters();
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(char * name) {
	type.assign(name);
	SetDefaultParameters();
	CheckParameters();
	AssignFunction();
}
RandomVariable::~RandomVariable() {
	// Nothing to do for now
}
RandomVariable::RandomVariable(string name, double x) {
	type=name; 
	a=x; 
	b=0.;
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(char * name, double x) {
	type.assign(name); 
	a=x; 
	b=0.;
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(string name, double x, double y) { 
	type=name; 
	a=x; 
	b=y;
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(char * name, double x, double y) {
	type.assign(name);
	a=x; 
	b=y;
	CheckParameters();
	AssignFunction();
}
double RandomVariable::GetValue() {
	return(function(a,b));
}
double RandomVariable::GetValue(RandomVariable *mere, double x) {
	return(nisp_pdfChange(this,mere,x));
}
void RandomVariable::GetLog() {
	ostringstream msg;
	msg << "***********************************************" << endl;
	msg << "Nisp(RandomVariable::GetLog) for RandomVariable" << endl;
	msg << type << " : " << a << " : " << b << endl;
	msg << "***********************************************" << endl;
	nisp_message ( msg.str() );
}

