// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2008-2009 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html
//
// Un petit exemple, adapt du document de spcification
//
// "Spcifications scientifiques et informatiques : Chaos Polynomial"
// D-WP1/08/01/A 
// MARTINEZ JM. - CEA
//

function y = Exemple (x)
  y(1) = x(1) * x(2);
endfunction
// 4.1 Reprsentation des paramtres incertains
vu1 = randvar_new("Normale",1.0,0.5);
vu2 = randvar_new("Uniforme",1.0,2.5);
srvu = setrandvar_new();
setrandvar_addrandvar ( srvu,vu1);
setrandvar_addrandvar ( srvu,vu2);
// Reprsentation des variables stochastiques
vx1 = randvar_new("Normale");
vx2 = randvar_new("Uniforme");
srvx = setrandvar_new();
setrandvar_addrandvar ( srvx,vx1);
setrandvar_addrandvar ( srvx,vx2);
ny = 1;
pc = polychaos_new ( srvx , ny );
// 4.3 Plan d'expriences numriques
degre = 2;
setrandvar_buildsample(srvx,"Quadrature",degre);
setrandvar_buildsample( srvu , srvx );
// 4.4 Ralisation du plan d'expriences numriques
np = setrandvar_getsize(srvx);
polychaos_setsizetarget(pc,np);
nx = polychaos_getdiminput(pc);
ny = polychaos_getdimoutput(pc);
for k=1:np
  inputdata = setrandvar_getsample(srvu,k);
  outputdata = Exemple(inputdata);
  polychaos_settarget(pc,k,outputdata);
end
// 4.5 Calcul des coefficients
polychaos_setdegree(pc,degre);
polychaos_computeexp(pc,srvx,"Integration");
// 4.6 Edition de l'analyse de sensibilit
average = polychaos_getmean(pc);
var = polychaos_getvariance(pc);
mprintf("Mean    = %f\n",average);
mprintf("Variance    = %f\n",var);
mprintf("Indice de sensibilit du 1er ordre\n");
mprintf("    Variable X1 = %f\n",polychaos_getindexfirst(pc,1));
mprintf("    Variable X2 = %f\n",polychaos_getindexfirst(pc,2));
mprintf("Indice de sensibilite Totale\n");
mprintf("    Variable X1 = %f\n",polychaos_getindextotal(pc,1));
mprintf("    Variable X2 = %f\n",polychaos_getindextotal(pc,2));
// Clean-up
polychaos_destroy(pc);
randvar_destroy(vu1);
randvar_destroy(vu2);
randvar_destroy(vx1);
randvar_destroy(vx2);
setrandvar_destroy(srvu);
setrandvar_destroy(srvx);

