
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

extern "C" {
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"
#include "gw_nisp.h"
}


/* ==================================================================== */

#include "nisp_gwsupport.hxx" 
#include "nisp_PolynomialChaos_map.hxx" 
#include "nisp_SetRandomVariable_map.hxx" 


/* ==================================================================== */
// 
// polychaos_getcorrelation ( token , ovar1 , ovar2 )
//   returns the correlation of the output variables which index are ovar1 and ovar2, 
//     where 1<= ovar1, ovar2 <= ny.
// polychaos_getcorrelation ( token )
//   returns the correlation of the all output variables, as a (ny) x (ny) matrix
//
int sci_polychaos_getcorrelation (char *fname)
{
	int nRows, nCols;
	int token;
	double *pdblFinalVar = NULL; //SCILAB return Var
	PolynomialChaos * pc = NULL;
	int ovar1 , ovar2;
	double correlation;
	int ny;
	double ** corrMatrix;

	CheckRhs(1,3) ;
	CheckLhs(0,1) ;
	// Get token
	nisp_gettoken ( fname , 1 , &token );
	if ( Rhs == 2 ) {
		Scierror(999,_("%s: Wrong number of input arguments: 1 or 3 expected.\n"),fname);
		return 1;
	}
	if ( Rhs == 3 ) {
		// polychaos_getcorrelation ( token , ovar1 , ovar2 )
		nisp_GetOneInteger ( fname , 2 , &ovar1 );
		nisp_GetOneInteger ( fname , 3 , &ovar2 );
	}
	nisp_token2Polychaos ( fname , 1 , token , &pc );
	if ( Rhs == 1 ) {
		// polychaos_getcorrelation ( token )
		ny = pc-> GetDimensionOutput();
		nRows=ny;
		nCols=ny;
		// Allocate a double ** and put the correlation matrix in it.
		corrMatrix = new double * [ny];
		for(int k = 0; k < ny; k++) {
			corrMatrix[k] = new double[ny];
		}
		pc -> GetCorrelation ( corrMatrix );
		// Copy the double ** into the double *
		iAllocMatrixOfDouble(Rhs+1, nRows, nCols, &pdblFinalVar);
		// The main loop is based on a line by line browsing of the output matrix,
		// which is stored in Fortran order, i.e. column by column.
		// For each line #k, the routine dcopy 
		//   * reads the variable sampleMatrix[k] element by element (i.e. INCX = 1), 
		//   * writes the variable pdblFinalVar is column by column (i.e. INCY = ny).
		// The first element of the line #k of the output matrix is at index pdblFinalVar+k.
		int INCX = 1;
		int INCY = ny;
		// Fill Scilab's matrix with sample, row by row
		for(int k = 0; k < ny; k++) {
			C2F(dcopy)(&ny,corrMatrix[k],&INCX,pdblFinalVar+k,&INCY);
		}
		for(int k = 0; k < ny; k++) {
			delete [] corrMatrix[k];
		}
		delete [] corrMatrix;
	} else if ( Rhs == 3 ) {
		// polychaos_getvariance ( token , ovar1 , ovar2 )
		correlation = pc -> GetCorrelation ( ovar1 , ovar2 );
		// Returns the result
		nRows=1;
		nCols=1;
		iAllocMatrixOfDouble(Rhs+1, nRows, nCols, &pdblFinalVar);
		pdblFinalVar[0] = correlation;
	}
	LhsVar(1) = Rhs+1;
	return 0;
}
/* ==================================================================== */

