
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

extern "C" {
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"
#include "gw_nisp.h"
}


/* ==================================================================== */

#include "nisp_gwsupport.hxx" 
#include "nisp_PolynomialChaos_map.hxx" 

extern int C2F(dcopy)();


/* ==================================================================== */
// 
// polychaos_getsample ( token , k , ovar )
//   returns the sample of experiment #k, where 1<= k<= np,
//   for output variable #ovar, where 1<= ovar<= ny
// polychaos_getsample ( token , k )
//   returns the sample of experiment #k, where 1<= k<= np,
//     as a (1) x ny matrix
// polychaos_getsample ( token )
//   returns the sampling as a (np) x ny matrix
//
int sci_polychaos_getsample (char *fname)
{
	int nRows, nCols;
	int token;
	double *pdblFinalVar = NULL; //SCILAB return Var
	PolynomialChaos * pc = NULL;
	int ovar;
	int k;
	int ny;
	int np;
	double ** sampleMatrix = NULL;

	CheckRhs(1,3) ;
	CheckLhs(0,1) ;
	nisp_gettoken ( fname , 1 , &token );
	if ( (Rhs==2) | (Rhs==3) ) {
		nisp_GetOneInteger ( fname , 2 , &k );
	}
	if (Rhs==3) {
		nisp_GetOneInteger ( fname , 3 , &ovar );
	}
	nisp_token2Polychaos ( fname , 1 , token , &pc );
	if (Rhs==1) {
		// polychaos_getsample ( token )
		int INCX;
		int INCY;
		ny = pc->GetDimensionOutput();
		np = pc->GetSampleSize( );
		sampleMatrix = new double * [np];
		for(int k = 0; k < np; k++) {
			sampleMatrix[k] = new double[ny];
		}
		pc->GetSample ( sampleMatrix );
		nRows=np;
		nCols=ny;
		iAllocMatrixOfDouble(Rhs+1, nRows, nCols, &pdblFinalVar);
		// The main loop is based on a line by line browsing of the output matrix,
		// which is stored in Fortran order, i.e. column by column.
		// For each line #k, the routine dcopy 
		//   * reads the variable sampleMatrix[k] element by element (i.e. INCX = 1), 
		//   * writes the variable pdblFinalVar is column by column (i.e. INCY = np).
		// The first element of the line #k of the output matrix is at index pdblFinalVar+k.
		INCX = 1;
		INCY = np;
		// Fill Scilab's matrix with sample, row by row
		for(int k = 0; k < np; k++) {
			C2F(dcopy)(&ny,sampleMatrix[k],&INCX,pdblFinalVar+k,&INCY);
		}
		for(int k = 0; k < np; k++) {
			delete [] sampleMatrix[k];
		}
		delete [] sampleMatrix;
	} else if (Rhs==2) {
		// polychaos_getquantile ( token , k )
		ny = pc-> GetDimensionOutput();
		// Returns the output as result
		nRows=1;
		nCols=ny;
		iAllocMatrixOfDouble(Rhs+1, nRows, nCols, &pdblFinalVar);
		pc-> GetSample ( k , pdblFinalVar );
	} else if (Rhs==3) {
		// polychaos_getsample ( token , k , ovar )
		// Returns the sample as result
		nRows=1;
		nCols=1;
		iAllocMatrixOfDouble(Rhs+1, nRows, nCols, &pdblFinalVar);
		pdblFinalVar[0] = pc-> GetSample ( k , ovar );
	}
	LhsVar(1) = Rhs+1;
	return 0;
}
/* ==================================================================== */

