
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

extern "C" {
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"
#include "gw_nisp.h"
}


/* ==================================================================== */

#include "nisp_gwsupport.hxx" 
#include "nisp_PolynomialChaos_map.hxx" 


/* ==================================================================== */
// 
// output = polychaos_gettarget ( token , k , j )
//   returns the output value for experiment #k and input #j, where k = 1, np and j=1,nx
// output = polychaos_gettarget ( token , k )
//   returns the output value for experiment #k and input #j, where k = 1, np
//   where output is a 1 x (nx) matrix
// output = polychaos_gettarget ( token )
//   returns the output matrix as a (np) x (nx) matrix
//
int sci_polychaos_gettarget (char *fname)
{
	int m, n;
	int token;
	double *pdblFinalVar = NULL; //SCILAB return Var
	PolynomialChaos * pc = NULL;
	double * mydata = NULL;
	int k;
	int j;
	int ny;
	double ** outputMatrix = NULL;
	int np;

	CheckRhs(1,3) ;
	CheckLhs(0,1) ;
	nisp_gettoken ( fname , 1 , & token );
	if ( ( Rhs == 2 ) | (Rhs == 3 ) ) {
		// output = polychaos_gettarget ( token , k )
		// output = polychaos_gettarget ( token , k , j )
		nisp_GetOneInteger ( fname , 2 , &k );
	}
	if ( Rhs == 3 ) {
		// output = polychaos_gettarget ( token , k , j )
		nisp_GetOneInteger ( fname , 3 , &j );
	}
	nisp_token2Polychaos ( fname , 1 , token , &pc );
	if ( Rhs == 1 ) {
		// output = polychaos_gettarget ( token )
		int INCX;
		int INCY;
		ny = pc -> GetDimensionOutput ();
		np = pc -> GetSizeTarget ();
		// Get target
		m=np;
		n=ny;
		iAllocMatrixOfDouble(Rhs+1, m, n, &pdblFinalVar);
		outputMatrix = new double * [np];
		for(int k = 0; k < np; k++) {
			outputMatrix[k] = new double[ny];
		}
		pc -> GetTarget ( outputMatrix );
		// The main loop is based on a line by line browsing of the output matrix,
		// which is stored in Fortran order, i.e. column by column.
		// For each line #k, the routine dcopy 
		//   * reads the variable sampleMatrix[k] element by element (i.e. INCX = 1), 
		//   * writes the variable pdblFinalVar is column by column (i.e. INCY = np).
		// The first element of the line #k of the output matrix is at index pdblFinalVar+k.
		INCX = 1;
		INCY = np;
		// Fill Scilab's matrix with sample, row by row
		for(int k = 0; k < np; k++) {
			C2F(dcopy)(&ny,outputMatrix[k],&INCX,pdblFinalVar+k,&INCY);
		}
		for(int k = 0; k < np; k++) {
			delete [] outputMatrix[k];
		}
		delete [] outputMatrix;
	} else if ( Rhs == 2 ) {
		ny = pc -> GetDimensionOutput ();
		// Get target
		m=1;
		n=ny;
		iAllocMatrixOfDouble(Rhs+1, m, n, &pdblFinalVar);
		pc -> GetTarget ( k , pdblFinalVar );
	} else if ( Rhs == 3 ) {
		// Get target
		m=1;
		n=1;
		iAllocMatrixOfDouble(Rhs+1, m, n, &pdblFinalVar);
		pdblFinalVar[0] = pc -> GetTarget ( k , j );
	}
	LhsVar(1) = Rhs+1;
	return 0;
}
/* ==================================================================== */

