
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

extern "C" {
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"
#include "gw_nisp.h"
}

/* ==================================================================== */

#include "nisp_gwsupport.hxx" 
#include "nisp_SetRandomVariable_map.hxx" 

extern int C2F(dcopy)();

/* ==================================================================== */
// 
// sample = setrandvar_getsample ( token , k , i ) 
//   returns the sample input #i=1, where 1<= i<= nx, where nx is the number 
//     of input random variables of the sample #k, where 1<= k<= np, where np is the number of experiments
// sample = setrandvar_getsample ( token , k )
//   returns the sample #k as a matrix with size 1 x (nx), where 1<= k<= np, np is the number of experiments and 
//      nx is the number of input random variables.
// sample = setrandvar_getsample ( token )
//   returns the sampling matrix with size (np) x (nx), where
//      np is the number of experiments and nx is the number of input random variables.
//
int sci_setrandvar_getsample (char *fname)
{
	int m, n;
	int token;
	SetRandomVariable * srv = NULL;
	double * mydata = NULL;
	double *pdblFinalVar = NULL; //SCILAB return Var
	int i, k;
	double ** sampleMatrix;
	int nx;
	int np;
	double sample;

	CheckRhs(1,3) ;
	CheckLhs(0,1) ;
	nisp_gettoken ( fname , 1 , & token );
	if ( Rhs == 3 ) {
		// sample = setrandvar_getsample ( token , k , i ) 
		nisp_GetOneInteger ( fname , 2 , &k );
		nisp_GetOneInteger ( fname , 3 , &i );
	} else if ( Rhs == 2 ) {
		// sample = setrandvar_getsample ( token , k )
		nisp_GetOneInteger ( fname , 2 , &k );
	}
	nisp_token2Setrandvar ( fname , 1 , token , &srv );
	if ( Rhs == 3 ) {
		// sample = setrandvar_getsample ( token , k , i ) 
		sample = srv->GetSample ( k , i );
		nisp_CreateLhsDouble ( 1 , sample );
	} else if ( Rhs == 2 ) {
		// sample = setrandvar_getsample ( token , k )
		nx = srv->GetDimension();
		m=1;
		n=nx;
		iAllocMatrixOfDouble(Rhs+1, m, n, &pdblFinalVar);
		srv->GetSample ( k , pdblFinalVar );
		LhsVar(1) = Rhs+1;
	} else if ( Rhs == 1 ) {
		int INCX;
		int INCY;
		// sample = setrandvar_getsample ( token )
		nx = srv->GetDimension();
		np = srv->GetSize();
		sampleMatrix = new double * [np];
		for(int k = 0; k < np; k++) {
			sampleMatrix[k] = new double[nx];
		}
		srv->GetSample ( sampleMatrix );
		m=np;
		n=nx;
		iAllocMatrixOfDouble(Rhs+1, m, n, &pdblFinalVar);
		// The main loop is based on a line by line browsing of the output matrix,
		// which is stored in Fortran order, i.e. column by column.
		// For each line #k, the routine dcopy 
		//   * reads the variable sampleMatrix[k] element by element (i.e. INCX = 1), 
		//   * writes the variable pdblFinalVar is column by column (i.e. INCY = np).
		// The first element of the line #k of the output matrix is at index pdblFinalVar+k.
		INCX = 1;
		INCY = np;
		// Fill Scilab's matrix with sample, row by row
		for(int k = 0; k < np; k++) {
			C2F(dcopy)(&nx,sampleMatrix[k],&INCX,pdblFinalVar+k,&INCY);
		}
		for(int k = 0; k < np; k++) {
			delete [] sampleMatrix[k];
		}
		delete [] sampleMatrix;
		LhsVar(1) = Rhs+1;
	}
	return 0;
}
/* ==================================================================== */

